#include "convertIIC.h"






int ConvertIIC(LA_HWInfo HWInfo, int nChannel, IICConfig IICCfg, unsigned char *cBuffer,
				LL_DataNode **dList)

{
	bool bExtendedDebug = false;

	char *msg;	
	msg = (char*)malloc(sizeof(char)*1000);

	if (bExtendedDebug)
	{
		
	//Konfiguration anzeigen
		#ifdef DebugMode
			sprintf(msg, "         ConvertIIC(); Begin (Clock: %d; Daten: %d\n", IICCfg.nChannelClk, IICCfg.nChannelData);	
			PrintDebug(msg);
		#endif
	}


	//Fehlerabfragen

	if ((HWInfo.bytes_per_sample*8) < HWInfo.channel_max) 
	{
		#ifdef DebugMode
			sprintf(msg, "                     (bytesPerSample*8) < ChannelMax\n");	
			PrintDebug(msg);
		#endif
		
		free(msg);
		return 1;
	}

	if (IICCfg.nChannelClk == IICCfg.nChannelData)
	{
		#ifdef DebugMode
			sprintf(msg, "                     IIC: Clk == Data\n");	
			PrintDebug(msg);
		#endif
		free(msg);
		return 1;
	}


	ULONG nSamplePos = 0;
	int nData, nClk,
		nLastData, nLastClk;


	//Zunchst: Warten, bis Data und CLK 1 sind

	while(nSamplePos < (HWInfo.samples_max ))
	{
		nData = IsSet(HWInfo,cBuffer,IICCfg.nChannelData, nSamplePos);
		nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);

		if (nData == -1 ||
			nClk == -1)
			return 0;

		if (nData == 1 &&
			nClk == 1)
		{
			break;
		}

		nSamplePos++;
	}

	bool bExtendedAddress = false;
	bool bWrite = false;
	bool bStopCondition = false;
	bool bRepeatedStartCondition = false;
	bool bAck;

	unsigned char byte[8];
	unsigned int nAddress;
	sData Data;
	char cStr[33], cVal[33];
	int i;

	nLastData = 0;
	nLastClk = 0;

	while (nSamplePos < (HWInfo.samples_max))
	{

		nData = IsSet(HWInfo,cBuffer,IICCfg.nChannelData, nSamplePos);
		nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);


		//Todo: Startsignal abwarten!

		// von 1:1 auf 1:0

		if ((nLastClk == 1 && nLastData == 1 &&
			nClk == 1 && nData == 0) ||
			bRepeatedStartCondition)
		{
			//Initialisierungen

			nAddress = 0;
			bWrite = false;
			bExtendedAddress = false;
			bAck = false;
			bStopCondition = false;
			bRepeatedStartCondition = false;

		#ifdef DebugMode
			sprintf(msg, "   [IIC] Start: %d\n", nSamplePos);	
			PrintDebug(msg);
		#endif
		



			//Warte, bis nClk auf 0 geht

			while (nSamplePos < (HWInfo.samples_max))
			{
				nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
				if (nClk == -1)
					return 0;
				if (nClk == 0)
					break;
				nSamplePos++;
			}

			Data.nStart = nSamplePos;

			//Adresse ( 4Bits HW-Adresse; 3Bits Sub-Adresse) + R/W lesen

			for (i=0;i<8;i++)
			{	
				//Warte, bis CLK == 1
				while (nSamplePos < (HWInfo.samples_max))
				{
					nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
					if (nClk == -1)
						return 0;
					if (nClk == 1)
						break;
					nSamplePos++;
				}

							
				//Datenbit einlesen (MSB zuerst)
				nData = IsSet(HWInfo,cBuffer,IICCfg.nChannelData, nSamplePos);

				if (nData == -1)
					return 0;

				byte[i] = nData;
			/*

			#ifdef DebugMode
				sprintf(msg, "   [IIC] ADDR(%d): %d\n", i, nData);	
				PrintDebug(msg);
			#endif
			*/
				//Warte, bis CLK == 0
				while (nSamplePos < (HWInfo.samples_max))
				{
					nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
					if (nClk == -1)
						return 0;
					if (nClk == 0)
						break;
					nSamplePos++;
				}
				
			}

			//ACK einlesen (SCL = 1; SDA = 0)

				//Warte, bis CLK == 1
				while (nSamplePos < (HWInfo.samples_max))
				{
					nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
					if (nClk == -1)
						return 0;
					if (nClk == 1)
						break;
					nSamplePos++;
				}

				nData = IsSet(HWInfo,cBuffer,IICCfg.nChannelData, nSamplePos);
				if (nData == -1)
					return 0;

				if (nData == 0)
					bAck = true;
				else
					bAck = false;

				//Warte, bis CLK == 0
				while (nSamplePos < (HWInfo.samples_max))
				{
					nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
					if (nClk == -1)
						return 0;
					if (nClk == 0)
						break;
					nSamplePos++;
				}


			//10 Bit Adressierung: Adresse= 11110XX

			if (byte[7] == 1)
				bWrite = false;
			else
				bWrite = true;

//zurzeit deaktiviert

			if (byte[0] == 1 &&
				byte[1] == 1 &&
				byte[2] == 1 &&
				byte[3] == 1 &&
				byte[4] == 0 && 0==1)
			{
				bExtendedAddress = true;

		#ifdef DebugMode
			sprintf(msg, "   [IIC] 10 Bit Addresse");	
			PrintDebug(msg);
		#endif

				//bit 5 und 6 verwerten!

				/*
				nAddress = nAddress | (byte[5] << 9);
				nAddress = nAddress | (byte[6] << 8);
				*/

				nAddress = nAddress | (byte[5] << 9);
				nAddress = nAddress | (byte[6] << 8);

				if (byte[5] == 1)
					cVal[0] = '1';
				else
					cVal[0] = '0';


				if (byte[6] == 1)
					cVal[1] = '1';
				else
					cVal[1] = '0';

				//weitere 8 Adress-bits:

				for (i=0;i<8;i++)
				{	
					//Warte, bis CLK == 1
					while (nSamplePos < (HWInfo.samples_max))
					{
						nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
						if (nClk == -1)
							return 0;
						if (nClk == 1)
							break;
						nSamplePos++;
					}
					
					//Datenbit einlesen (MSB zuerst)
					nData = IsSet(HWInfo,cBuffer,IICCfg.nChannelData, nSamplePos);

					if (nData == -1)
						return 0;

					byte[i] = nData;

					/*
			#ifdef DebugMode
				sprintf(msg, "    - [IIC] Byte(%d): %d\n", i,byte[i]);	
				PrintDebug(msg);
			#endif
					*/

					//Warte, bis CLK == 0
					while (nSamplePos < (HWInfo.samples_max))
					{
						nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
						if (nClk == -1)
							return 0;
						if (nClk == 0)
							break;
						nSamplePos++;
					}
					
				}

				//ACK einlesen

				//Warte, bis CLK == 1
				while (nSamplePos < (HWInfo.samples_max))
				{
					nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
					if (nClk == -1)
						return 0;
					if (nClk == 1)
						break;
					nSamplePos++;
				}

				nData = IsSet(HWInfo,cBuffer,IICCfg.nChannelData, nSamplePos);
				if (nData == -1)
					return 0;

				if (nData == 0)
					bAck = true;
				else
					bAck = false;

				//Warte, bis CLK == 0
				while (nSamplePos < (HWInfo.samples_max))
				{
					nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
					if (nClk == -1)
						return 0;
					if (nClk == 0)
						break;
					nSamplePos++;
				}



				for (i=0;i<8;i++)
				{
					nAddress = nAddress | (byte[i] << (7-i));
					if (byte[i] == 1)
						cVal[2+i] = '1';
					else
						cVal[2+i] = '0';
				}

				cVal[10] = '\0';


			}
			else
			{
				for (int i=0;i<7;i++)
				{
					nAddress = nAddress | (byte[i] << (6-i));
					if (byte[i] == 1)
						cVal[i] = '1';
					else
						cVal[i] = '0';
				}
				cVal[7] = '\0';
			}


			//Packet: Addresse Abschlieen

			#ifdef DebugMode
				sprintf(msg, "   [IIC] ADDR: %d ( %s ) ", nAddress, cVal);	

				if (bWrite)
					strcat(msg, " [W]");
				else
					strcat(msg, " [R]");

				if (bAck)
					strcat(msg, "  -ACK\n");
				else
					strcat(msg, "  -NACK\n");
				PrintDebug(msg);
			#endif


			switch (IICCfg.cType)
			{
				case TYPE_DEZ:
					itoa(nAddress, cVal, 10);
					break;
				case TYPE_HEX:
					itoa(nAddress, cVal, 16);
					break;
				case TYPE_BIN:
					//cVal liegt schon binr vor
					break;
			}




			Data.nLength = nSamplePos - Data.nStart;
			Data.cChannel = nChannel;
			Data.cText = 0;
			Data.cColor = 0;
			Data.cType = IICCfg.cType;

			strncpy(cStr, "Adr.: ",33);
			strncat(cStr, cVal,33);
			if (bWrite)
				strncat(cStr, "  [W]",33);
			else
				strncat(cStr, "  [R]",33);

			if (bAck)
				strncat(cStr, " [Ack]",33);
			else
				strncat(cStr, " [No-Ack]", 33);
			strncpy(Data.cStr, cStr,33);
			
			//Datenpaket hinzufgen

			(*dList) = LL_DataAdd((*dList), Data);

			//Datenpackete einlesen, falls ACK und kein Stop-Signal

			while(!bStopCondition && !bRepeatedStartCondition && bAck == true)
				{
				//9 Takte einlesen
				//falls Takt[0] == Stopbedinung, wird kein Datenpaket mehr gesendet
				//Takt[0] kann jedoch auch eine repeatet Start-Condition sein!

				bAck = false;
				Data.nStart = nSamplePos +1;
				nAddress = 0;	//Wird hier fr den Datenwert wverwendet
				for (i=0;i<8;i++)
				{	
					//Warte, bis CLK == 1
					while (nSamplePos < (HWInfo.samples_max))
					{
						nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
						if (nClk == -1)
							return 0;
						if (nClk == 1)
							break;
						nSamplePos++;
					}
					//eine Pos. nach SLK == 1
					nSamplePos++;
								
					//Datenbit einlesen (MSB zuerst)
					nData = IsSet(HWInfo,cBuffer,IICCfg.nChannelData, nSamplePos);

					if (nData == -1)
						return 0;

					byte[i] = nData;

				/*
				#ifdef DebugMode
					sprintf(msg, "   [IIC] Data(%d): %d   (Pos: %d)\n", i, nData, nSamplePos);	
					PrintDebug(msg);
				#endif
				*/
					//Warte, bis CLK == 0
					//ODER: (stop-condition) SDA != nData
					int nTmpData;
					while (nSamplePos < (HWInfo.samples_max))
					{
						nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
						nTmpData = IsSet(HWInfo,cBuffer,IICCfg.nChannelData, nSamplePos);

						if (nClk == -1)
							return 0;
						if (nClk == 0)
							break;

						//Stop-Condition:
						if (nTmpData == 1 && nData == 0)
						{
				#ifdef DebugMode
					sprintf(msg, "   [IIC] Stop Condition");	
					PrintDebug(msg);
				#endif

							bStopCondition = true;
							break;
						}

						//Start-Condition:
						if (nTmpData == 0 && nData == 1)
						{
						
				#ifdef DebugMode
					sprintf(msg, "   [IIC] Repeated Start Condition");	
					PrintDebug(msg);
				#endif
							bRepeatedStartCondition = true;
							break;
						}

						nSamplePos++;
					}

					if (bStopCondition || bRepeatedStartCondition)
						break;

					
				}

				if (bStopCondition)
					break;
				if (bRepeatedStartCondition)
					break;

				//ACK/NACK

					//Warte, bis CLK == 1
					while (nSamplePos < (HWInfo.samples_max))
					{
						nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
						if (nClk == -1)
							return 0;
						if (nClk == 1)
							break;
						nSamplePos++;
					}

					nData = IsSet(HWInfo,cBuffer,IICCfg.nChannelData, nSamplePos);
					if (nData == -1)
						return 0;

					if (nData == 0)
						bAck = true;
					else
						bAck = false;



					//Warte, bis CLK == 0
					while (nSamplePos < (HWInfo.samples_max))
					{
						nClk = IsSet(HWInfo,cBuffer,IICCfg.nChannelClk, nSamplePos);
						if (nClk == -1)
							return 0;
						if (nClk == 0)
							break;
						nSamplePos++;
					}

				//Datenpaket hinzufgen
					
					for (int i=0;i<8;i++)
					{
						nAddress = nAddress | (byte[i] << (7-i));
						if (byte[i] == 1)
							cVal[i] = '1';
						else
							cVal[i] = '0';
					}
					cVal[8] = '\0';

				#ifdef DebugMode
					sprintf(msg, "   [IIC] Data: %d ( %s ) ", nAddress, cVal);	
					if (bAck)
						strcat(msg, "      -ACK");
					else
						strcat(msg, "      -NACK");

					PrintDebug(msg);
				#endif

					

					switch (IICCfg.cType)
					{
						case TYPE_DEZ:
							itoa(nAddress, cVal, 10);
							break;
						case TYPE_HEX:
							itoa(nAddress, cVal, 16);
							break;
						case TYPE_BIN:
							//cVal liegt schon binr vor
							break;
					}


					Data.nLength = nSamplePos - Data.nStart;
					Data.cChannel = nChannel;
					Data.cText = 0;
					Data.cColor = 0;
					Data.cType = IICCfg.cType;

					strncpy(cStr, "Data: ",33);
					strncat(cStr, cVal,33);

					if (bAck)
						strncat(cStr, " [Ack]",33);
					else
						strncat(cStr, " [No-Ack]", 33);
					strncpy(Data.cStr, cStr,33);
					
					//Datenpaket hinzufgen


					(*dList) = LL_DataAdd((*dList), Data);

			}



		}


		nLastClk = nClk;
		nLastData = nData;

		nSamplePos ++;
	}



	free(msg);

	return 0;
}